"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const lhisp_oauth_client_1 = require("../src/lhisp-oauth-client");
const lhisp_oauth_client_t_1 = require("../src/lhisp-oauth-client.t");
// Mock jest and set the type
jest.mock("axios");
const mockedAxios = axios_1.default;
const apiUrl = "https://myapi.com";
const authUrl = "https://auth.myapi.com/oauth/token";
const clientId = "testClientdId";
const clientSecret = "testClientSecret";
const baseClientParams = { apiUrl, authUrl, clientId, clientSecret };
const basicAuth = `Basic ${Buffer.from(`${clientId}:${clientSecret}`).toString("base64")}`;
const contentTypeApplicationJson = lhisp_oauth_client_t_1.ContentType.APPLICATION_JSON;
const contentTypeApplicationXFormUrlEncoded = lhisp_oauth_client_t_1.ContentType.APPLICATION_X_WWW_FORM_URLENCODED;
const defaultGrantValue = "client_credentials";
const defaultGrantType = `grant_type=${defaultGrantValue}`;
describe("Get Access Token", () => {
    beforeEach(() => {
        mockedAxios.request.mockReset();
        mockedAxios.request.mockResolvedValueOnce({ data: mockedAccessToken });
    });
    it("Shoud Get with Standard Config", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient();
        yield accessTokenValidator(cli);
        validateDefaultGetAccessToken();
    }));
    it("Shoud Get with Custom Auth Header", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient(Object.assign(Object.assign({}, baseClientParams), { authHeaderName: "CustomAuthorizationHeader" }));
        yield accessTokenValidator(cli);
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: authUrl,
            method: "POST",
            headers: {
                CustomAuthorizationHeader: basicAuth,
                "Content-Type": contentTypeApplicationXFormUrlEncoded,
            },
            data: defaultGrantType,
        }));
    }));
    it("Shoud Get with Custom Grant Type", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient(Object.assign(Object.assign({}, baseClientParams), { grantType: "PermissaoCustom" }));
        yield accessTokenValidator(cli);
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: authUrl,
            method: "POST",
            headers: {
                Authorization: basicAuth,
                "Content-Type": contentTypeApplicationXFormUrlEncoded,
            },
            data: "grant_type=PermissaoCustom",
        }));
    }));
    it("Shoud Get with Custom Auth Scope", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient(Object.assign(Object.assign({}, baseClientParams), { authScope: "EscopoCustom" }));
        yield accessTokenValidator(cli);
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: authUrl,
            method: "POST",
            headers: {
                Authorization: basicAuth,
                "Content-Type": contentTypeApplicationXFormUrlEncoded,
            },
            data: `grant_type=${defaultGrantValue}&scope=EscopoCustom`,
        }));
    }));
    it("Shoud Get with Credentials on Request body", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient(Object.assign(Object.assign({}, baseClientParams), { authContentType: contentTypeApplicationJson, sendAuthCredentialsOnRequestBody: true }));
        yield accessTokenValidator(cli);
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: authUrl,
            method: "POST",
            headers: {
                Authorization: basicAuth,
                "Content-Type": contentTypeApplicationJson,
            },
            data: `{"grant_type":"${defaultGrantValue}","client_id":"${clientId}","client_secret":"${clientSecret}"}`,
        }));
    }));
});
describe("Request", () => {
    beforeEach(() => {
        mockedAxios.request.mockReset();
        mockedAxios.request.mockResolvedValueOnce({ data: mockedAccessToken });
        mockedAxios.request.mockResolvedValueOnce({ data: { status: "ok" } });
    });
    it("Get without Params", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient();
        const resp = yield cli.get({ path: "/my-test-url" });
        validateDefaultGetAccessToken();
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: `${apiUrl}/my-test-url`,
            method: "GET",
            headers: {
                Authorization: `Bearer SomeAccessToken`,
                "Content-Type": contentTypeApplicationJson,
            },
            data: undefined,
        }));
        expect(resp).toStrictEqual({ status: "ok" });
    }));
    it("Get with Params", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient();
        const resp = yield cli.get({ path: "/my-test-url", params: { id: 1 } });
        validateDefaultGetAccessToken();
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: `${apiUrl}/my-test-url`,
            method: "GET",
            headers: {
                Authorization: `Bearer SomeAccessToken`,
                "Content-Type": contentTypeApplicationJson,
            },
            params: { id: 1 },
            data: undefined,
        }));
        expect(resp).toStrictEqual({ status: "ok" });
    }));
    it("Post", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient();
        const resp = yield cli.post({ path: "/my-test-url-post", data: { id: 1, user: "test" } });
        validateDefaultGetAccessToken();
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: `${apiUrl}/my-test-url-post`,
            method: "POST",
            headers: {
                Authorization: `Bearer SomeAccessToken`,
                "Content-Type": contentTypeApplicationJson,
            },
            data: { id: 1, user: "test" },
        }));
        expect(resp).toStrictEqual({ status: "ok" });
    }));
    it("Post with different contentType", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient();
        const resp = yield cli.post({
            path: "/my-test-url-post",
            data: { id: 1, user: "test" },
            contentType: lhisp_oauth_client_t_1.ContentType.APPLICATION_X_WWW_FORM_URLENCODED,
        });
        validateDefaultGetAccessToken();
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: `${apiUrl}/my-test-url-post`,
            method: "POST",
            headers: {
                Authorization: `Bearer SomeAccessToken`,
                "Content-Type": contentTypeApplicationXFormUrlEncoded,
            },
            data: { id: 1, user: "test" },
        }));
        expect(resp).toStrictEqual({ status: "ok" });
    }));
    it("Post with Different Token Header Name", () => __awaiter(void 0, void 0, void 0, function* () {
        const cli = getOauthClient(Object.assign(Object.assign({}, baseClientParams), { tokenHeaderName: "x-token" }));
        const resp = yield cli.post({
            path: "/my-test-url-post",
            data: { id: 1, user: "test" },
            contentType: lhisp_oauth_client_t_1.ContentType.APPLICATION_X_WWW_FORM_URLENCODED,
        });
        validateDefaultGetAccessToken();
        expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
            url: `${apiUrl}/my-test-url-post`,
            method: "POST",
            headers: {
                "x-token": `Bearer SomeAccessToken`,
                "Content-Type": contentTypeApplicationXFormUrlEncoded,
            },
            data: { id: 1, user: "test" },
        }));
        expect(resp).toStrictEqual({ status: "ok" });
    }));
});
function validateDefaultGetAccessToken() {
    expect(mockedAxios.request).toBeCalledWith(expect.objectContaining({
        url: authUrl,
        method: "POST",
        headers: {
            Authorization: basicAuth,
            "Content-Type": contentTypeApplicationXFormUrlEncoded,
        },
        data: defaultGrantType,
    }));
}
function accessTokenValidator(cli) {
    return __awaiter(this, void 0, void 0, function* () {
        const now = Date.now();
        const accessToken = yield cli.getAccessToken();
        expect(accessToken).toBeDefined();
        expect(accessToken.token_type).toBe(mockedAccessToken.token_type);
        expect(accessToken.access_token).toBe(mockedAccessToken.access_token);
        expect(accessToken.expires_in).toBe(mockedAccessToken.expires_in);
        expect(accessToken.scope).toBe(mockedAccessToken.scope);
    });
}
function getOauthClient(opt = baseClientParams) {
    return new lhisp_oauth_client_1.LhispOauthClient(opt);
}
const mockedAccessToken = {
    token_type: "Bearer",
    access_token: "SomeAccessToken",
    expires_in: 600,
    scope: "cobrancas.boletos-requisicao cobrancas.boletos-info",
};
