/// <reference types="node" />
/// <reference types="node" />
import https from "https";
import { AxiosHeaders } from "axios";
import { AccessToken, ContentType, ExecutarRequestParams, LhispOauthClientConstructorParams } from "./lhisp-oauth-client.t";
export declare class LhispOauthClient<iAccessToken extends AccessToken = AccessToken> {
    protected authUrl: string;
    protected apiUrl: string;
    protected clientId: string;
    protected clientSecret: string;
    protected authHeaderName: string;
    protected tokenHeaderName: string;
    protected authContentType: ContentType;
    protected certificado?: string | Buffer;
    protected senhaCertificado?: string;
    protected authScope?: string;
    protected headers?: AxiosHeaders;
    protected authHeaders?: AxiosHeaders;
    protected grantType?: string;
    protected agent: https.Agent;
    protected authData: Record<string, string>;
    protected accessToken?: iAccessToken;
    protected refreshToken?: iAccessToken;
    protected tokenCreatedAt: number;
    protected tokenExpiresIn: number;
    protected sendAuthCredentialsOnRequestBody?: boolean;
    protected timeout: number;
    protected formatAccessToken?: (accessToken?: iAccessToken) => string;
    constructor(params: LhispOauthClientConstructorParams);
    getAuthHeaderValue(): string;
    parseData({ data, contentType }: {
        data: any;
        contentType: string;
    }): string | undefined;
    isTokenValid(): boolean;
    getAccessToken(): Promise<iAccessToken>;
    buildAccessToken(data: any): iAccessToken;
    getAuthToken(): string;
    executarRequest<ResponseType>({ method, path, data, params, contentType, ...opt }: ExecutarRequestParams): Promise<ResponseType>;
    get<ResponseType>(opt: ExecutarRequestParams): Promise<ResponseType>;
    put<ResponseType>(opt: ExecutarRequestParams): Promise<ResponseType>;
    patch<ResponseType>(opt: ExecutarRequestParams): Promise<ResponseType>;
    post<ResponseType>(opt: ExecutarRequestParams): Promise<ResponseType>;
    delete<ResponseType>(opt: ExecutarRequestParams): Promise<ResponseType>;
}
