"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LhispOauthClient = void 0;
const querystring_1 = __importDefault(require("querystring"));
const https_1 = __importDefault(require("https"));
const axios_1 = __importDefault(require("axios"));
const lhisp_oauth_client_t_1 = require("./lhisp-oauth-client.t");
const lhisp_logger_1 = __importDefault(require("lhisp-logger"));
class LhispOauthClient {
    constructor(params) {
        this.authData = {};
        this.tokenCreatedAt = 0;
        this.tokenExpiresIn = 0;
        if (params.certificado) {
            this.agent = new https_1.default.Agent({
                pfx: Buffer.isBuffer(params.certificado) ? params.certificado : Buffer.from(params.certificado, "base64"),
                passphrase: params.senhaCertificado ? params.senhaCertificado : undefined,
                rejectUnauthorized: false,
            });
        }
        else {
            this.agent = new https_1.default.Agent({
                rejectUnauthorized: false,
            });
        }
        this.certificado = params.certificado;
        this.headers = (params.headers ? params.headers : {});
        this.authHeaders = (params.authHeaders ? params.authHeaders : {});
        this.authData = params.authData || {};
        this.apiUrl = params.apiUrl;
        this.authUrl = params.authUrl;
        this.authScope = params.authScope;
        this.grantType = params.grantType || lhisp_oauth_client_t_1.defaultGrantType;
        this.authContentType = params.authContentType || lhisp_oauth_client_t_1.defaultAuthContentType;
        this.clientId = params.clientId;
        this.clientSecret = params.clientSecret;
        this.authHeaderName = params.authHeaderName || lhisp_oauth_client_t_1.defaultAuthHeaderName;
        this.tokenHeaderName = params.tokenHeaderName || lhisp_oauth_client_t_1.defaultTokenHeaderName;
        this.sendAuthCredentialsOnRequestBody = params.sendAuthCredentialsOnRequestBody;
        this.formatAccessToken = params.formatAccessToken;
        this.timeout = params.timeout ? params.timeout : 60000;
    }
    getAuthHeaderValue() {
        return `Basic ${Buffer.from(`${this.clientId}:${this.clientSecret}`).toString("base64")}`;
    }
    parseData({ data, contentType = lhisp_oauth_client_t_1.ContentType.APPLICATION_JSON }) {
        if (!data || Object.keys(data).length === 0)
            return undefined;
        switch (contentType) {
            case lhisp_oauth_client_t_1.ContentType.APPLICATION_JSON:
                return JSON.stringify(data);
            case lhisp_oauth_client_t_1.ContentType.APPLICATION_X_WWW_FORM_URLENCODED:
                return querystring_1.default.stringify(data);
            default:
                throw new Error(`Content Type Inválido: [${contentType}]`);
        }
    }
    isTokenValid() {
        if (!this.accessToken)
            return false;
        if (!this.tokenCreatedAt)
            return false;
        const timeDiff = (Date.now() - this.tokenCreatedAt) / 1000;
        return timeDiff < this.tokenExpiresIn - 10;
    }
    getAccessToken() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.accessToken && this.isTokenValid()) {
                    return this.accessToken;
                }
                // TODO: Implementar Refresh Token.
                let authRequestOpt = {
                    method: "POST",
                    url: this.authUrl,
                    httpsAgent: this.agent,
                    headers: Object.assign({ [this.authHeaderName]: this.getAuthHeaderValue(), "Content-Type": this.authContentType }, this.authHeaders),
                    data: Object.assign({}, this.authData),
                    timeout: this.timeout,
                };
                if (this.grantType)
                    authRequestOpt.data.grant_type = this.grantType;
                if (this.authScope)
                    authRequestOpt.data.scope = this.authScope;
                if (this.sendAuthCredentialsOnRequestBody) {
                    if (this.clientId)
                        authRequestOpt.data.client_id = this.clientId;
                    if (this.clientSecret)
                        authRequestOpt.data.client_secret = this.clientSecret;
                }
                authRequestOpt.data = this.parseData({
                    data: authRequestOpt.data,
                    contentType: this.authContentType,
                });
                const resp = yield axios_1.default.request(authRequestOpt);
                this.accessToken = this.buildAccessToken(resp.data);
                this.tokenCreatedAt = new Date().getTime();
                this.tokenExpiresIn = ((_a = this.accessToken) === null || _a === void 0 ? void 0 : _a.expires_in) || this.tokenCreatedAt + 60000;
                return this.accessToken;
            }
            catch (error) {
                lhisp_logger_1.default.error({ message: "LhispOauthClient.getAccessToken", error });
                throw error;
            }
        });
    }
    buildAccessToken(data) {
        return data;
    }
    getAuthToken() {
        var _a, _b;
        if (this.formatAccessToken) {
            return this.formatAccessToken(this.accessToken);
        }
        return `${(_a = this.accessToken) === null || _a === void 0 ? void 0 : _a.token_type} ${(_b = this.accessToken) === null || _b === void 0 ? void 0 : _b.access_token}`;
    }
    executarRequest(_a) {
        var { method, path, data, params, contentType = lhisp_oauth_client_t_1.ContentType.APPLICATION_JSON } = _a, opt = __rest(_a, ["method", "path", "data", "params", "contentType"]);
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.getAccessToken();
                const headers = Object.assign({ "Content-Type": contentType, [this.tokenHeaderName]: this.getAuthToken() }, (this.headers || {}));
                const response = yield axios_1.default.request(Object.assign({ method, url: `${this.apiUrl}${path}`, httpsAgent: this.agent, headers,
                    data,
                    params, timeout: this.timeout }, opt));
                return response.data;
            }
            catch (error) {
                lhisp_logger_1.default.error({
                    message: "LhispOauthClient.executarRequest",
                    method,
                    url: `${this.apiUrl}${path}`,
                    data,
                    params,
                    contentType,
                    error,
                });
                throw error;
            }
        });
    }
    get(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executarRequest(Object.assign({ method: "GET" }, opt));
        });
    }
    put(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executarRequest(Object.assign({ method: "PUT" }, opt));
        });
    }
    patch(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executarRequest(Object.assign({ method: "PATCH" }, opt));
        });
    }
    post(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executarRequest(Object.assign({ method: "POST" }, opt));
        });
    }
    delete(opt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.executarRequest(Object.assign({ method: "DELETE" }, opt));
        });
    }
}
exports.LhispOauthClient = LhispOauthClient;
